/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target;

import java.io.InputStream;
import java.net.URL;

import org.eclipse.core.runtime.*;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.target.internal.TargetPlugin;

/**
 * A handle to a resource that is (or may be) located on a site 
 * (e.g. local file system, FTP or WebDAV). The state of the resource
 * located on the site can be queried and modified through the
 * methods of this interface.
 * 
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under 
 * development and expected to change significantly before reaching stability. 
 * It is being made available at this early stage to solicit feedback from pioneering 
 * adopters on the understanding that any code that uses this API will almost 
 * certainly be broken (repeatedly) as the API evolves.
 * </p>
 */
public interface ITargetResource extends IAdaptable {
	
	/**
	 * Constant that acts as the prefix of the properties defined in this interface
	 */
	String PROPERTY_PREFIX = TargetPlugin.ID;
	
	/**
	 * Constant that is used to fetch and access the size of a resource.
	 * The size is a <code>long</code> which can be accessed using the
	 * {@link #getSize()} method.
	 */
	QualifiedName SIZE_PROPERTY = new QualifiedName(PROPERTY_PREFIX, "size"); //$NON-NLS-1$
	
	/**
	 * Constant used to fetch and access the last modified timestamp of the resource handle.
	 * The timestamp is a <code>String</code> and can be accessed using {@link #getProperty(QualifiedName, IProgressMonitor)}.
	 */
	QualifiedName LAST_MODIFIED_TIMESTAMP_PROPERTY = new QualifiedName(PROPERTY_PREFIX, "lastModified"); //$NON-NLS-1$
	
	/**
	 * Constant used to fetch and access the creator name of the resource handle.
	 * The name is a <code>String</code> and can be accessed using {@link #getProperty(QualifiedName, IProgressMonitor)}.
	 */
	QualifiedName CREATOR_DISPLAY_NAME_PROPERTY = new QualifiedName(PROPERTY_PREFIX, "creator"); //$NON-NLS-1$
	
	/**
	 * Constant used to fetch and access the content identifier of the resource handle.
	 * The content identifier should uniquely identify the contents of a resource. 
	 * Given the non-transactional nature of most site types, the identifer
	 * may be the last-modified-timestamp combined with the size.
	 * The content identifier is a <code>String</code> and can be accessed using {@link #getProperty(QualifiedName, IProgressMonitor)}.
	 */
	QualifiedName CONTENT_IDENTIFIER_PROPERTY = new QualifiedName(PROPERTY_PREFIX, "content"); //$NON-NLS-1$

	/**
	 * Return the site where this resource is located.
	 * @return the site where this resource is located
	 */
	Site getSite();
	
	/**
	 * Return the name of this resource. The name is the last segment
	 * of the resource's path. An empty string is returned if this resource
	 * represents the root of it's site's file system.
	 * @return the name of this resource or an empty string if the resource
	 * is at the root of it's site's file system
	 */
	String getName();

	/**
	 * Return the full path of this resource. A portion
	 * of the full path will identify the site while
	 * the remainder will be the site relative path.
	 * @return the full path of this resource
	 */
	String getFullPath();
	
	/**
	 * Return the path of this resource relative to its 
	 * site. The path will be the empty string if this resource
	 * is the root of the site.
	 * @return the path of this resource relative to its 
	 * site
	 */
	String getRelativePath();
	
	/**
	 * Return the parent directory of this resource
	 * or <code>null</code> if the resource is the root
	 * of the site.
	 * @return the parent directory of this resource
	 * or <code>null</code> if the resource is the root
	 * of the site
	 */
	ITargetResource getParent();

	/**
	 * Return a URL that uniquely identifies this resource
	 * or <code>null</code> if the site does not support
	 * URL resource identification.
	 * @return the URL that uniquely identifies this resource
	 * (may be <code>null</code>)
	 */
	URL toUrl();
	
	/**
	 * Return whether the resource represented by this
	 * handle is a directory or a file.
	 * @return whether the resource represented by this
	 * handle is a directory or a file.
	 */
	boolean isDirectory();
	
	
	/**
	 * Return a boolean value indicating whether or not this resource exists on the
	 * remote server.
	 * @param monitor a progress monitor
	 */
	public boolean exists(IProgressMonitor monitor) throws CoreException;
	
	/**
	 * Answers and array of <code>ITargetResource</code> elements that are immediate 
	 * children of this target resource, in no particular order. For files, this
	 * method should always return an empty array. For directories, an array of
	 * zero or more items will be returned.
	 * 
 	 * @param progress a progress monitor to indicate the duration of the operation
	 * @return array of immediate children of this target resource. 
	 */
	ITargetResource[] members(IProgressMonitor monitor) throws CoreException;

	/**
	 * Returns a stream over the contents of this remote element. For directories,
	 * an empty stream is returned.
	 * 
 	 * @param progress a progress monitor to indicate the duration of the operation.
	 */
	public InputStream getContents(IProgressMonitor progress) throws CoreException;
	
	/**
	 * Set the contents of the file represented by the resource handle to
	 * the contents of the provided stream. The resource must either not
	 * exist or must already be a file.
	 * @param stream a stream that contains the new contents for the file.
	 * @param size the number of bytes in the new contents
	 * @param progress a progress monitor
	 * @throws CoreException
	 */
	public void setContents(InputStream stream, long size, IProgressMonitor progress) throws CoreException;
	
	/**
	 * Deletes the resource represented by this resource handle.
	 * @param monitor a progress monitor
	 * @throws CoreException
	 */
	public void delete(IProgressMonitor monitor) throws CoreException;
	
	/**
	 * Fetch the requested properties of the resource identified by this
	 * handle. If refresh is <code>false</code> the properties will only
	 * be fetched if this handle does not have values cached already. 
	 * If refresh is <code>true</code>, the properties will be fetched from
	 * the site, regardless of cahced properties.
	 * @param properties
	 * @param refresh fetch the properties even if the handle already has values cached.
	 * @param monitor a progress monitor
	 * @return the properties that are now available in the handle.
	 * @throws CoreException
	 */
	public QualifiedName[] fetchProperties(QualifiedName[] properties, boolean refresh, IProgressMonitor monitor) throws CoreException;
	
	/**
	 * Return whether the given property is available in this handle.
	 * A return value of <code>true</code> indicates that the property
	 * can be accessed without first calling <code>fetchProperties</code>.
	 * Otherwise, <code>fetchProperties</code> must be invoked with the
	 * property as an argument in order to fetch the property from the 
	 * site.
	 * @param property the property being queried
	 * @return whether the property is available in the handle or must be
	 * fetched from the server using <code>fetchProperties</code>
	 */
	public boolean isAvailable(QualifiedName property);
	
	/**
	 * Return whether this resource supports the given property.
	 * A return value of <code>false</code> indicates that the property
	 * will be ignored when fetched and that the value returned by the
	 * method associated with the property will be meaningless.
	 * @param property the property being queried
	 * @return whether this resource supports the given property
	 */
	public boolean isSupported(QualifiedName property);
	
	/**
	 * Return an array of the properties that can be associated
	 * with this resource handle.
	 * @return an array of the properties that can be associated
	 * with this resource handle.
	 */
	public QualifiedName[] getSupportedProperties();
	
	/**
	 * Return the requested property of the resource represented
	 * by this handle. Callers must ensure that the property is
	 * available before invoking this method, either by calling 
	 * {@link #isAvailable(QualifiedName)} or 
	 * {@link #fetchProperties(QualifiedName[], boolean)}.
	 * An empty string is returned if the property is not available.
	 * @param property the property being requested
	 * @return the size of the resource or an empty string.
	 */
	String getProperty(QualifiedName property) throws CoreException;

	/**
	 * Return the size of the resource represented by this handle.
	 * Callers must ensure that the handle has the SIZE_PROPERTY
	 * available before invoking this method, either by calling 
	 * {@link #isAvailable(QualifiedName)} or 
	 * {@link #fetchProperties(QualifiedName[], boolean)}.
	 * A size of 0 is returned if the property is not available.
	 * @return the size of the resource or 0.
	 */
	long getSize();

	/**
 	 * Returns a handle to the remote file identified by the given path in this
 	 * folder.
	 * <p> 
	 * This is a remote resource handle operation; neither the resource nor
	 * the result need exist on the server.</p>
	 * <p>
	 * The supplied path may be absolute or relative; in either case, it is
	 * interpreted as relative to this resource and is appended
	 * to this container's full path to form the full path of the resultant resource.
	 * A trailing separator is ignored.
	 * </p>
	 *
	 * @param name the path of the remote member file
	 * @return the (handle of the) remote file
	 * @see #getFolder
	 */
	public ITargetResource getFile(String name);

	/**
 	 * Returns a handle to the remote folder identified by the given path in this
 	 * folder.
	 * <p> 
	 * This is a remote resource handle operation; neither the resource nor
	 * the result need exist on the server.</p>
	 * <p>
	 * The supplied path may be absolute or relative; in either case, it is
	 * interpreted as relative to this resource and is appended
	 * to this container's full path to form the full path of the resultant resource.
	 * A trailing separator is ignored.
	 * </p>
	 *
	 * @param path the path of the remote member file
	 * @return the (handle of the) remote file
	 * @see #getFolder
	 */
	public ITargetResource getFolder(String name);

	/**
	 * Creates the directory on the site that is represented by this 
	 * resource handle.
	 * @param monitor a progress monitor
	 */
	void mkdir(IProgressMonitor monitor) throws CoreException;
	
	/**
	 * Creates the directory on the site that is represented by this 
	 * resource handle, including any necessary but non-existant
	 * parent directories. 
	 * @param monitor a progress monitor
	 */
	void mkdirs(IProgressMonitor monitor) throws CoreException;

}
