/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target.internal.ui;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;
import org.eclipse.target.ITargetResource;
import org.eclipse.target.Target;

/**
 * Dialog which allows the user to select a target folder
 */
public class TargetFolderSelectionDialog extends Dialog {

	private TreeViewer folderTree;
	private ITargetResource selected;
	private final IEnablementChecker checker;

	public interface IEnablementChecker {
		boolean isEnabledFor(ITargetResource folder);
	}
	public TargetFolderSelectionDialog(Shell parentShell, IEnablementChecker checker) {
		super(parentShell);
		if (checker == null) {
			checker = new IEnablementChecker() {
				public boolean isEnabledFor(ITargetResource folder) {
					return true;
				}
			};
		}
		this.checker = checker;
	}
	
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText("Copy Resources");
	} 
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) {
		Composite c = (Composite)super.createDialogArea(parent);
		
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		initializeDialogUnits(c);
		layout.marginHeight = convertVerticalDLUsToPixels(IDialogConstants.VERTICAL_MARGIN);
		layout.marginWidth = convertHorizontalDLUsToPixels(IDialogConstants.HORIZONTAL_MARGIN);
		c.setLayout(layout);
		
		Label l = new Label(c, SWT.NONE);
		l.setText("Choose the target folder for the copy");
		l.setData(new GridData(GridData.FILL_HORIZONTAL));
		
		folderTree = SiteExplorerView.createTargetFolderViewer(c, false);
		GridData data = new GridData(GridData.FILL_BOTH);
		data.heightHint = 200;
		data.widthHint = 200;
		folderTree.getControl().setLayoutData(data);
		folderTree.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				selected = null;
				if (!event.getSelection().isEmpty()) {
					selected = getSelectedFolder(event.getSelection());
				}
				getButton(IDialogConstants.OK_ID).setEnabled(selected != null);
			}
		});
		folderTree.setInput(Target.getSiteManager());
		return c;
	}
	
	protected ITargetResource getSelectedFolder(ISelection selection) {
		IStructuredSelection ss = (IStructuredSelection)selection;
		Object o = ss.getFirstElement();
		ITargetResource resource = null;
		if (o instanceof ITargetResource) {
			resource = (ITargetResource)o;
		} else if (o instanceof IAdaptable) {
			IAdaptable adaptable = (IAdaptable)o;
			resource = (ITargetResource)adaptable.getAdapter(ITargetResource.class);
		}
		if (resource != null && resource.isDirectory() && checker.isEnabledFor(resource)) {
			return resource;
		}
		return null;
	}
	
	public ITargetResource getSelectedFolder() {
		return selected;
	}

}
